@@ -1,11 +1,15 @@
 """
+The pythonbible library.
+
 pythonbible includes features for parsing texts for scripture references,
 converting references into integer verse ids for efficient use/storage,
 converting verse ids back into normalized references, and formatting
 references as human-readable strings.
 """
 
-__version__ = "0.7.1"
+from __future__ import annotations
+
+__version__ = "0.7.3"
 
 from .bible.osis.parser import OSISParser
 from .book_groups import BOOK_GROUPS, BookGroup
@@ -47,8 +51,8 @@ from .verses import (
     get_book_chapter_verse,
     get_book_number,
     get_chapter_number,
-    get_number_of_verses,
     get_number_of_chapters,
+    get_number_of_verses,
     get_verse_id,
     get_verse_number,
 )
